package hirondelle.web4jtools.codegenerator.action;

import java.util.*;
import hirondelle.web4j.model.AppException;
import hirondelle.web4j.action.ActionImpl;
import hirondelle.web4j.request.RequestParser;
import hirondelle.web4j.action.ResponsePage;
import hirondelle.web4jtools.util.TemplatedPage;
import hirondelle.web4jtools.codegenerator.codes.UiStyle;
import hirondelle.web4jtools.codegenerator.feature.Feature;
import hirondelle.web4jtools.codegenerator.feature.FeatureAction;
import hirondelle.web4jtools.codegenerator.field.Field;
import hirondelle.web4jtools.codegenerator.field.FieldAction;
import hirondelle.web4jtools.util.Functions;
import javax.servlet.ServletConfig;
import hirondelle.web4jtools.util.Ensure;

/**
* Generate the source code of your feature's Action class, as plain text.
* 
* <P>Uses the {@link hirondelle.web4jtools.codegenerator.feature.Feature} and 
* List of {@link hirondelle.web4jtools.codegenerator.field.Field} objects placed in session scope.
*/
public final class ActionAction extends ActionImpl {
  
  /**
  * Read in necessary config from <tt>web.xml</tt>.
  * 
  * Called only upon startup.
  */
  public static void readConfig(ServletConfig aConfig){
    fTEMPLATED_PAGE_CLASS = aConfig.getInitParameter(TEMPLATED_PAGE_CLASS);
    Ensure.isPresentInWebXml(TEMPLATED_PAGE_CLASS, fTEMPLATED_PAGE_CLASS);
  }
  
  public ActionAction(RequestParser aRequestParser){
    super(LIST_AND_EDIT, aRequestParser);  
  }

  /** Generate the Action class. */
  @Override public ResponsePage execute() throws AppException {
    UiStyle uiStyle = getUiStyle();
    if ( UiStyle.ListAndEdit == uiStyle ) {
      //use default forward
    }
    else if ( UiStyle.ShowAndApply == uiStyle ) {
      setResponsePage(SHOW_AND_APPLY);
    }
    else {
      throw new AssertionError("Unexpected value for UiStyle : " + uiStyle);
    }
    
    addPKIdentifierToRequest();
    addImportForTemplatedPage();
    
    return getResponsePage();
  }
  
  // PRIVATE //
  private static final ResponsePage LIST_AND_EDIT = TemplatedPage.getPlain("Action", "listAndEdit.jsp", ActionAction.class);
  private static final ResponsePage SHOW_AND_APPLY = TemplatedPage.getPlain("Action", "showAndApply.jsp", ActionAction.class);
  
  private static final String PK_KEY = "primaryKey";
  
  private static final String TEMPLATED_PAGE_CLASS = "TemplatedPageClass";
  private static String fTEMPLATED_PAGE_CLASS = "";
  
  private UiStyle getUiStyle() {
    Feature feature = (Feature)getFromSession(FeatureAction.FEATURE_KEY);
    return feature.getUiStyle();
  }

  private void addPKIdentifierToRequest(){
    boolean isPkPresent = false;
    List<Field> fields = (List<Field>) getFromSession(FieldAction.FIELDS_KEY);
    for(Field field : fields){
      if( field.getIsPrimaryKey() ) {
        isPkPresent = true;
        addToRequest(PK_KEY, Functions.asConstant(field.getName().toString()));
        break;
      }
    }
    if ( ! isPkPresent ) {
      addError("No field is marked as being Primary Key.");
      addToRequest(PK_KEY, "ID");      
    }
  }
  
  private void addImportForTemplatedPage(){
    addToRequest(TEMPLATED_PAGE_CLASS, fTEMPLATED_PAGE_CLASS);
  }
}
