package hirondelle.web4jtools.codegenerator.feature;

import hirondelle.web4j.action.ActionTemplateShowAndApply;
import hirondelle.web4j.action.ResponsePage;
import hirondelle.web4j.model.AppException;
import hirondelle.web4j.model.ModelCtorException;
import hirondelle.web4j.model.ModelFromRequest;
import hirondelle.web4j.request.RequestParameter;
import hirondelle.web4j.request.RequestParser;

/** 
* Edit your feature's basic info.
*/
public final class FeatureAction extends ActionTemplateShowAndApply {

  /** Constructor. */
  public FeatureAction(RequestParser aRequestParser){
    super(FORWARD, REDIRECT, aRequestParser);
  }
  
  public static final RequestParameter FEATURE_NAME = RequestParameter.withLengthCheck("Name");
  public static final RequestParameter UI_STYLE = RequestParameter.withLengthCheck("UiStyle");
  public static final RequestParameter PACKAGE_NAME = RequestParameter.withLengthCheck("PackageName");
  public static final RequestParameter SHOW_OPERATION = RequestParameter.withLengthCheck("ShowOperation");
  public static final RequestParameter APPLY_OPERATION = RequestParameter.withLengthCheck("ApplyOperation");
  
  /** If no feature name has been input, then shows a blank form.  */
  @Override  protected void show() throws AppException {
    addToRequest(ITEM_FOR_EDIT, fetchFeature());
  }

  /** Check user input can build a {@link Feature} object.  */
  @Override protected void validateUserInput() throws AppException {
    try {
      ModelFromRequest builder = new ModelFromRequest(getRequestParser());
      fFeature = builder.build(Feature.class, FEATURE_NAME, UI_STYLE, PACKAGE_NAME, SHOW_OPERATION, APPLY_OPERATION);
    }
    catch (ModelCtorException ex){
      addError(ex);
    }
  }  

  /** Save the {@link Feature} Model Object input by the user.  */
  @Override protected void apply() throws AppException {
    saveFeature();
    addMessage("'_1_' feature updated.", fFeature.getName());
  }

  /** Key for storing Feature object in session scope.  */
  public static final String FEATURE_KEY = "feature";

  // PRIVATE //
  private Feature fFeature;
  private static final ResponsePage FORWARD = new ResponsePage("Feature", "view.jsp", FeatureAction.class);
  private static final ResponsePage REDIRECT = new ResponsePage("FeatureAction.do?Operation=Show");

  /** Returns null if no {@link Feature} is in the session.  */
  private Feature fetchFeature(){
    return (Feature)getFromSession(FEATURE_KEY);
  }
  
  /** Save {@link Feature} in session scope.  */
  private void saveFeature(){
    addToSession(FEATURE_KEY, fFeature);
  }
}
