package hirondelle.web4jtools.codegenerator.jsp;

import hirondelle.web4j.model.AppException;
import hirondelle.web4j.action.ActionImpl;
import hirondelle.web4j.request.RequestParser;
import hirondelle.web4j.action.ResponsePage;
import hirondelle.web4jtools.codegenerator.codes.UiStyle;
import hirondelle.web4jtools.codegenerator.feature.Feature;
import hirondelle.web4jtools.codegenerator.feature.FeatureAction;
import hirondelle.web4jtools.util.TemplatedPage;

/** 
* Generate a JSP snippet for the feature, and serve as plain text. 
* 
* <P>Uses the {@link hirondelle.web4jtools.codegenerator.feature.Feature} and 
* List of {@link hirondelle.web4jtools.codegenerator.field.Field} objects placed in session scope.
*/
public final class JspAction extends ActionImpl {

  public JspAction(RequestParser aRequestParser){
    super(LIST_AND_EDIT, aRequestParser);
  }
  
  @Override  public ResponsePage execute() throws AppException {
    UiStyle uiStyle = getUiStyle();
    if ( UiStyle.ListAndEdit == uiStyle ) {
      //use default forward
    }
    else if ( UiStyle.ShowAndApply == uiStyle ) {
      setResponsePage(SHOW_AND_APPLY);
    }
    else {
      throw new AssertionError("Unexpected value for UiStyle : " + uiStyle);
    }
    return getResponsePage();
  }
  
  // PRIVATE //
  private static final ResponsePage LIST_AND_EDIT = TemplatedPage.getPlain("JSP", "listAndEdit.jsp", JspAction.class);
  private static final ResponsePage SHOW_AND_APPLY = TemplatedPage.getPlain("JSP", "showAndApply.jsp", JspAction.class);
  
  private UiStyle getUiStyle() {
    Feature feature = (Feature) getFromSession(FeatureAction.FEATURE_KEY);
    return feature.getUiStyle();
  }
}
