package hirondelle.web4jtools.codegenerator.output;

import hirondelle.web4j.action.ActionImpl;
import hirondelle.web4j.action.ResponsePage;
import hirondelle.web4j.model.AppException;
import hirondelle.web4j.request.RequestParser;
import hirondelle.web4jtools.codegenerator.feature.FeatureAction;
import hirondelle.web4jtools.codegenerator.field.FieldAction;
import hirondelle.web4jtools.util.SaveToFile;

/**
* Present links for generating your feature's source code.
* 
* <P>When you click on each link, the corresponding source code is generated and served as plain text.
* There are two ways to place the generated source code into your development environment:
*<ul>
* <li>a manual copy and paste operation
* <li>automatic file generation, if enabled by settings in <tt>web.xml</tt>. This will create files 
* directly in your development directory, as a side-effect of serving the plain text.
* See the overview for more information on this option.
*</ul>
*
*<P>If the necessary items are not found in session scope, then an error message is displayed as a warning.
*/
public final class OutputAction extends ActionImpl {
  
  public OutputAction(RequestParser aRequestParser){
    super(FORWARD, aRequestParser);
  }
  
  @Override public ResponsePage execute() throws AppException {
    if ( SaveToFile.isEnabled() ) {
      addMessage("Clicking on these links will save a file to the server's file system.");
    }
    ensurePresentInSession(FeatureAction.FEATURE_KEY);
    ensurePresentInSession(FieldAction.FIELDS_KEY);
    return getResponsePage();
  }
  
  // PRIVATE //
  private static final ResponsePage FORWARD =  new ResponsePage("Select Output", "view.jsp", OutputAction.class);

  private void ensurePresentInSession(String aKey){
    Object object = getFromSession(aKey);
    if ( object == null ) {
      addError("Required item is not in session : " + aKey + ". Please enter before proceeding with generating output.");
    }
  }
}
