package hirondelle.web4jtools.logview.directories;

import hirondelle.web4j.action.ActionTemplateShowAndApply;
import hirondelle.web4j.action.ResponsePage;
import hirondelle.web4j.model.AppException;
import hirondelle.web4j.model.ModelCtorException;
import hirondelle.web4j.model.ModelFromRequest;
import hirondelle.web4j.request.RequestParameter;
import hirondelle.web4j.request.RequestParser;

/**
* Allow user to override settings in <tt>web.xml</tt> for {@link LogInfo}. 
*/
public final class LogInfoAction extends ActionTemplateShowAndApply {

  public LogInfoAction(RequestParser aRequestParser){
    super(FORWARD, REDIRECT, aRequestParser);
  }
  
  public static final RequestParameter PROJECT_NAME = RequestParameter.withLengthCheck("ProjectName");  
  public static final RequestParameter APP_LOGGING_DIR = RequestParameter.withLengthCheck("AppLoggingDirectory");  
  public static final RequestParameter SERVER_LOGGING_DIR = RequestParameter.withLengthCheck("ServerLoggingDirectory");  
  public static final RequestParameter SERVER_LOG_FILE_STARTS_WITH = RequestParameter.withLengthCheck("ServerLogFileStartsWith");  
  
  /** Show a form with current settings. */
  @Override protected void show() throws AppException {
    LogInfoDAO dao  = new LogInfoDAO(getRequestParser().getRequest());
    addToRequest(ITEM_FOR_EDIT, dao.fetch());
  }
  
  /** Validate changes to current settings.  */
  @Override protected void validateUserInput() throws AppException {
    ModelFromRequest builder = new ModelFromRequest(getRequestParser());
    try {
      fLogInfo = builder.build(LogInfo.class, PROJECT_NAME, APP_LOGGING_DIR, SERVER_LOGGING_DIR, SERVER_LOG_FILE_STARTS_WITH);
    }
    catch (ModelCtorException ex) {
      addError(ex);
    }
  }
  
  /** Save the changes to current settings. */
  @Override protected void apply() throws AppException {
    LogInfoDAO dao  = new LogInfoDAO(getRequestParser().getRequest());
    dao.save(fLogInfo);
  }
  
  // PRIVATE //
  private LogInfo fLogInfo;
  private static final ResponsePage FORWARD = new ResponsePage("Logging Directories", "view.jsp", LogInfoAction.class);
  private static final ResponsePage REDIRECT = new ResponsePage("LogInfoAction.do?Operation=Show");
}
