package hirondelle.web4jtools.logview.parser;

import hirondelle.web4jtools.logview.parsedview.LoggerRecord;
import hirondelle.web4jtools.logview.parsedview.ParsedCriteria;
import java.util.List;

/**
* Generic interface for parsing an entire log file into specific records.
* 
* <P>A single implementation of this interface is provided, which parses 
* logs in the format emitted by default by JDK loggers. This is the 
* default format for both the WEB4J framework classes, and the popular 
* Tomcat server.
* 
* <P>If your application or server uses a different log file format, then you 
* will need to 
*<ul>
* <li>supply a new implementation of this interface
* <li>amend the implementation of {@link hirondelle.web4jtools.logview.parser.LogParserInstance}
*</ul>
*/
public interface LogParser {

  /**
  * Parse the contents of a log file into {@link LoggerRecord} objects.
  * 
  * <P><em>Design Note</em><br>
  * This interface does not use streams. Streams would likely be more 
  * performant in handling large files. However, since this tool is meant as 
  * a <em>development</em> tool, and not for production (because of 
  * security concerns), this is usually acceptable, since during development 
  * log files rarely become excessively large.
  */
  List<LoggerRecord> parse(String aLogFileContents, ParsedCriteria aCriteria);
  
}
