package hirondelle.web4jtools.metrics.comments;

import java.util.*;
import hirondelle.web4j.request.RequestParser;
import hirondelle.web4j.action.ResponsePage;
import javax.servlet.ServletConfig;
import hirondelle.web4jtools.util.Ensure;
import hirondelle.web4jtools.metrics.base.FileInfo;
import hirondelle.web4jtools.metrics.base.MetricsAction;
import java.util.Comparator;
import java.util.List;


/**
* List java classes with a low number of comments.
* 
* <P>Calculate the overall percentage of comments in your code.
* Uses the <tt>ThresholdForLowComments</tt> setting in <tt>web.xml</tt>.
*/
public final class CommentsAction extends MetricsAction {

  /**  Read in settings from <tt>web.xml</tt>.   */
  public static void readConfig(ServletConfig aConfig){
    String rawConfig = aConfig.getInitParameter(THRESHOLD_LOW_COMMENTS);
    Ensure.isPresentInWebXml(THRESHOLD_LOW_COMMENTS, rawConfig);
    fTHRESHOLD_LOW_COMMENTS = Long.valueOf(rawConfig);
    if ( fTHRESHOLD_LOW_COMMENTS < 0 || 100 < fTHRESHOLD_LOW_COMMENTS) {
      throw new IllegalArgumentException(THRESHOLD_LOW_COMMENTS + " in web.xml is not in range 0..100.");
    }
  }
  
  public CommentsAction(RequestParser aRequestParser){
    super(FORWARD, aRequestParser);
  }
  
  /** Calculate metrics and show list of classes with low numbers of comments. */
  protected void calculateMetric() {
    addToRequest("commentsThreshold", fTHRESHOLD_LOW_COMMENTS);
    scanFiles(fFileList);
    addToRequest("numLines", fNumLines);
    addToRequest("numCommentLines",  fNumCommentLines);
    Collections.sort(fFilesWithLowComments, byPercent());
    addToRequest("filesWithLowComments", fFilesWithLowComments);
  }
  
  // PRIVATE //
  private long fNumLines;
  private long fNumCommentLines;
  private List<FileInfo> fFilesWithLowComments = new ArrayList<FileInfo>();
  private static final String THRESHOLD_LOW_COMMENTS = "ThresholdForLowComments"; 
  private static Long  fTHRESHOLD_LOW_COMMENTS; 
  private static final ResponsePage FORWARD = new ResponsePage("Comment Frequency", "view.jsp", CommentsAction.class);
  
  private void scanFiles(List<FileInfo> aFiles){
    for(FileInfo file : aFiles){
      if (file.isJavaSourceFile()) {
        fNumLines = file.getNumLines();
        fNumCommentLines = file.getNumCommentLines();
        if( lowInComments(file)){
          fFilesWithLowComments.add(file);
        }
      }
    }
  }
  
  private boolean lowInComments(FileInfo aFile){
    return  aFile.getPercentComments()  < fTHRESHOLD_LOW_COMMENTS;
  }
  
  private Comparator<FileInfo> byPercent(){
    class Result implements Comparator<FileInfo> {
      public int compare(FileInfo aThis, FileInfo aThat) {
        return aThis.getPercentComments().compareTo(aThat.getPercentComments());
      }
    }
    return new Result();
  }
}
