package hirondelle.web4jtools.metrics.histogram;

/**
* Model Object for an element of a histogram, whose elements have
* numeric ranges.
* 
* <P>This class is mutable.
*/
public final class Bucket {

  /**
  * Full constructor.
  * 
  * <P>Min must be less than max.
  */
  public Bucket(int aMin, int aMax){
    fMin = aMin;
    fMax = aMax;
    validateState();
  }
  
  /** Special constructor for the 'last' bucket, which has no upper limit.   */
  public Bucket(Integer aMin) {
    fMin = aMin;
    fMax = Integer.MAX_VALUE;
  }

  /** Increase the number of items in this bucket by one. */
  public void incrementCount(){ ++fCount; }
  /** Return the number of items in this bucket.  */
  public Integer getCount(){ return fCount; }
  
  public Integer getMin(){  return fMin; }  
  public Integer getMax() {
    return fMax == Integer.MAX_VALUE ? null : fMax;
  }
  
  // PRIVATE //
  private int fMin;
  private int fMax;
  private int fCount;
  
  private void validateState() {
    if ( fMin >= fMax ) {
      throw new IllegalArgumentException("Min must me less than max. Min :" + fMin + " Max : " + fMax);
    }
  }
}
