package hirondelle.web4jtools.metrics.images;

import java.util.*;
import hirondelle.web4j.model.Check;
import hirondelle.web4j.model.ModelUtil;
import static hirondelle.web4j.util.Consts.FAILS;
import hirondelle.web4jtools.metrics.base.FileInfo;
import hirondelle.web4j.util.Util;

/** 
* Model Object for an image, and all files that reference it.
* <P>This class is mutable. 
*/
public final class ImageReference {

  /**
  * Full constructor.
  *  
  * @param aFileInfo file info for the image file.
  */
  public ImageReference(FileInfo aFileInfo) {
    fFileInfo = aFileInfo;
    validateState();
  }
  
  /** Add a file which references the image.  */
  public void addReferenceFrom(FileInfo aFileInfo) {
    fReferencingFiles.add(aFileInfo);
  }
  
  /** Return the file information for the image itself.  */
  public FileInfo getFileInfo() { return fFileInfo; }
  
  /** List all files that <em>reference</em> the image. */
  public List<FileInfo> getReferencingFiles(){ return fReferencingFiles; }
  
  /** Return the number of files that reference the image. */
  public Integer getNumReferences(){
    return fReferencingFiles.size();
  }
  
  @Override public String toString(){
    return ModelUtil.toStringFor(this);
  }
  
  @Override public boolean equals(Object aThat){
    Boolean result = ModelUtil.quickEquals(this, aThat);
    if ( result == null ) {
      ImageReference that = (ImageReference)aThat;
      result = ModelUtil.equalsFor(this.getSignificantFields(), that.getSignificantFields());
    }
    return result;
  }
  
  @Override public int hashCode(){
    //no caching - mutable
    return ModelUtil.hashCodeFor(getSignificantFields());
  }

  // PRIVATE //
  private final FileInfo fFileInfo;
  private final List<FileInfo> fReferencingFiles = new ArrayList<FileInfo>();

  private void validateState() {
    StringBuilder errorMsg = new StringBuilder();
    if ( FAILS == Check.required(fFileInfo) ) {
      errorMsg.append("Name is required.");
    }
    if ( Util.textHasContent(errorMsg.toString()) ) {
      throw new IllegalArgumentException(errorMsg.toString());
    }
  }
  
  private Object[] getSignificantFields(){
    return new Object[] {fFileInfo, fReferencingFiles};
  }
}