package hirondelle.web4jtools.metrics.tabs;

import hirondelle.web4j.action.ResponsePage;
import hirondelle.web4j.request.RequestParser;
import hirondelle.web4jtools.metrics.base.FileInfo;
import hirondelle.web4jtools.metrics.base.MetricsAction;

import java.util.ArrayList;
import java.util.List;

/**
* List source files containing at least one tab character. 
*
* <P>Since tools often display tabs with excessively large indentation, tab characters 
* are usually undesirable in code.
*/
public class TabsListAction extends MetricsAction {

  /** Full constructor. */
  public TabsListAction(RequestParser aRequestParser) {
    super(FORWARD, aRequestParser);
  }

  /**
  * List source files containing one or more TAB characters.  
  * 
  *<P>In addition, show these stats : 
  *<ul>
  * <li>total number of tab characters in source code
  * <li>total number of source code files containing at least one tab character
  *</ul>
  */
  @Override  protected void calculateMetric() {
    List<FileInfo> filesWithTabs = filterFilesWithTabs();
    addToRequest("filesWithTabs", filesWithTabs);
    addToRequest("numTabs", fNumTabs);
  }
  
  // PRIVATE //
  private int fNumTabs;
  private static final ResponsePage FORWARD = new ResponsePage("Files With TAB Characters", "view.jsp", TabsListAction.class);

  private List<FileInfo> filterFilesWithTabs(){
    List<FileInfo> result = new ArrayList<FileInfo>();
    for(FileInfo fileInfo : fFileList){
      if ( fileInfo.getNumTabs() > 0 ) {
        result.add(fileInfo);
        fNumTabs = fNumTabs + fileInfo.getNumTabs();
      }
    }
    return result;
  }
}
