package hirondelle.web4jtools.util;

import java.io.IOException;
import javax.servlet.jsp.JspException;
import hirondelle.web4j.ui.tag.TagHelper;
import hirondelle.web4j.util.Util;
import static hirondelle.web4j.util.Consts.NOT_FOUND;

/**
* Trim the final comma appearing in text, along with any trailing characters appearing after 
* the final comma.
* 
* <P>This tag is used in code generation of lists of items separated by a comma.
* It is almost always necessary to remove the final comma from such lists. 
*/
public final class LastCommaTag extends TagHelper {

  @Override protected String getEmittedText(String aText) throws JspException, IOException {
    String result = aText;
    if (  Util.textHasContent(aText) ) {
      int finalCommaIdx = aText.lastIndexOf(COMMA);
      if ( isFound(finalCommaIdx) ){
        result = removeFinalComma(aText, finalCommaIdx);
      }
    }
    return result;
  }
  
  // PRIVATE //
  private static final String COMMA = ",";
  
  private boolean isFound(int aFinalComma){
    return aFinalComma != NOT_FOUND;
  }
  
  private String removeFinalComma(String aText, int aFinalCommaIdx){
    return aText.substring(0, aFinalCommaIdx);
  }
}
